#include "TNR_HEX_MESH.hxx"
#include <map>
#include <vector>
//#include <windows.h>



// get positon that use uv parameter
TNR_POINT  TNR_solid8_get_position(TNR_POINT* c, 
								    double  u,
									double  v,
									double  t)
{


 double xi   = ( u * 2.0 ) - 1.0;
 double eta  = ( v * 2.0 ) - 1.0;
 double zeta = ( t * 2.0 ) - 1.0;

 double N0 = ( ( 1.0 + xi ) * ( 1.0 - eta ) * (1.0 - zeta) ) / 8.0;
 double N1 = ( ( 1.0 + xi ) * ( 1.0 + eta ) * (1.0 - zeta) ) / 8.0;
 double N2 = ( ( 1.0 - xi ) * ( 1.0 + eta ) * (1.0 - zeta) ) / 8.0;
 double N3 = ( ( 1.0 - xi ) * ( 1.0 - eta ) * (1.0 - zeta) ) / 8.0;
 double N4 = ( ( 1.0 + xi ) * ( 1.0 - eta ) * (1.0 + zeta) ) / 8.0;
 double N5 = ( ( 1.0 + xi ) * ( 1.0 + eta ) * (1.0 + zeta) ) / 8.0;
 double N6 = ( ( 1.0 - xi ) * ( 1.0 + eta ) * (1.0 + zeta) ) / 8.0;
 double N7 = ( ( 1.0 - xi ) * ( 1.0 - eta ) * (1.0 + zeta) ) / 8.0;

 TNR_POINT p = N0*c[0] + N1*c[1] + N2*c[2] + N3*c[3] + N4*c[4] + N5*c[5] + N6*c[6] + N7*c[7];

 return p;

}

// get positon that use uv parameter
TNR_POINT  TNR_solid20_get_position(TNR_POINT* c, 
								    double  pu,
									double  pv,
									double  pt)
{


 double u  = ( pu * 2.0 ) - 1.0;
 double v  = ( pv * 2.0 ) - 1.0;
 double t  = ( pt * 2.0 ) - 1.0;

 double N0 = 0.125 *  ( 1.0 - u ) * ( 1.0 - v ) * (1.0 - t) * (-2.0 - u - v - t);
 double N1 = 0.125 *  ( 1.0 + u ) * ( 1.0 - v ) * (1.0 - t) * (-2.0 + u - v - t);
 double N2 = 0.125 *  ( 1.0 + u ) * ( 1.0 + v ) * (1.0 - t) * (-2.0 + u + v - t);
 double N3 = 0.125 *  ( 1.0 - u ) * ( 1.0 + v ) * (1.0 - t) * (-2.0 - u + v - t);
 double N4 = 0.125 *  ( 1.0 - u ) * ( 1.0 - v ) * (1.0 + t) * (-2.0 - u - v + t);
 double N5 = 0.125 *  ( 1.0 + u ) * ( 1.0 - v ) * (1.0 + t) * (-2.0 + u - v + t);
 double N6 = 0.125 *  ( 1.0 + u ) * ( 1.0 + v ) * (1.0 + t) * (-2.0 + u + v + t);
 double N7 = 0.125 *  ( 1.0 - u ) * ( 1.0 + v ) * (1.0 + t) * (-2.0 - u + v + t);
 double N8 = 0.25  *  ( 1.0 - u*u ) * ( 1.0 - v ) * (1.0 - t);
 double N9 = 0.25  *  ( 1.0 + u ) * ( 1.0 - v*v ) * (1.0 - t);
 double N10 = 0.25  *  ( 1.0 - u*u ) * ( 1.0 + v ) * (1.0 - t);
 double N11 = 0.25  *  ( 1.0 - u ) * ( 1.0 - v*v ) * (1.0 - t);
 double N12 = 0.25  *  ( 1.0 - u ) * ( 1.0 - v) * (1.0 - t*t);
 double N13 = 0.25  *  ( 1.0 + u ) * ( 1.0 - v) * (1.0 - t*t);
 double N14 = 0.25  *  ( 1.0 + u ) * ( 1.0 + v) * (1.0 - t*t);
 double N15 = 0.25  *  ( 1.0 - u ) * ( 1.0 + v) * (1.0 - t*t);
 double N16 = 0.25  *  ( 1.0 - u*u ) * ( 1.0 - v ) * (1.0 + t);
 double N17 = 0.25  *  ( 1.0 + u ) * ( 1.0 - v*v ) * (1.0 + t);
 double N18 = 0.25  *  ( 1.0 - u*u ) * ( 1.0 + v ) * (1.0 + t);
 double N19 = 0.25  *  ( 1.0 - u ) * ( 1.0 - v*v ) * (1.0 + t);

 TNR_POINT p = N0*c[0] + N1*c[1] + N2*c[2] + N3*c[3] + N4*c[4] + N5*c[5] + N6*c[6] + N7*c[7] + N8*c[8] + N9*c[9]
               + N10*c[10]+ N11*c[11]+ N12*c[12]+ N13*c[13]+ N14*c[14]+ N15*c[15]+ N16*c[16] + N17*c[17] + N18*c[18] + N19*c[19];

 return p;

}


void TNR_hex_create_mesh(TNR_SECTION* body,
							 TNR_COMAMND* commands,
							 TNR_HEX_MESH* mesh)
{
  //get mesh parameter
  int  udiv = commands->getIntValue("u_div",1);
  int  vdiv = commands->getIntValue("v_div",1);
  int  tdiv = commands->getIntValue("t_div",1);

   //initilize control points
  TNR_POINT ctlPoints[20];

  ctlPoints[0].set(1.0, 0.0, 0.0);
  ctlPoints[1].set(1.0, 1.0, 0.0); 
  ctlPoints[2].set(0.0, 1.0, 0.0); 
  ctlPoints[3].set(0.0, 0.0, 0.0); 
  ctlPoints[4].set(1.0, 0.0, 1.0); 
  ctlPoints[5].set(1.0, 1.0, 1.0); 
  ctlPoints[6].set(0.0, 1.0, 1.0);
  ctlPoints[7].set(0.0, 0.0, 1.0); 


  //in case of cube8,create control points
  string interpolationType = body->getStringValue("interpolation_type","");
  if(interpolationType == "cube8" || interpolationType == "cube20"){
    int numPoins = body->getNumData();
    for(int i = 0; i < numPoins; i++){
      string data = body->getData(i);
	  printf("string data = %s\n",data.c_str());
      sscanf(data.c_str(),"%lf %lf %lf", &(ctlPoints[i][0]),&(ctlPoints[i][1]),&(ctlPoints[i][2]));
	}
  }

  //in case of cube,create control points
  if(interpolationType == "cube"){
    TNR_POINT scale;
    scale[0] = body->getDoubleValue("side_length",1.0);
    scale[1] = body->getDoubleValue("height_length",1.0);
    scale[2] = body->getDoubleValue("depth_length",1.0);

    //scale ctrl points
    for(int i = 0; i < 8;i++){
      ctlPoints[i] = ctlPoints[i] * scale;
    }
  }



  //initilize
  int numMeshPoints = (udiv+1)*(vdiv+1)*(tdiv+1);
  int numMeshQuads = udiv*vdiv*tdiv;
  mesh->initilize(numMeshPoints,numMeshQuads);

  int uloop = udiv + 1;
  int vloop = vdiv + 1;
  int tloop = tdiv + 1;

  int posCount = 0;
  double uIncre = 1.0 / udiv;
  double vIncre = 1.0 / vdiv;
  double tIncre = 1.0 / tdiv;
  double uPos = 0.0;
  double vPos = 0.0;
  double tPos = 0.0;

  //create points
  for(int t = 0; t < tloop; t++){
     vPos = 0.0;	 
     for(int v = 0; v < vloop; v++){
       uPos = 0.0;	  
       for(int u = 0; u < uloop; u++){

	     if(interpolationType == "cube" || interpolationType == "cube8"  ){
		    mesh->setPoint(posCount,TNR_solid8_get_position(ctlPoints,uPos,vPos,tPos));
	     }
	     if(interpolationType == "cube20"){
	        mesh->setPoint(posCount,TNR_solid20_get_position(ctlPoints,uPos,vPos,tPos));
	     }
         printf("param(u,v,t): %f %f %f\n",uPos,vPos,tPos);
         uPos += uIncre;
	     posCount++;
	   }
	   vPos += vIncre;
	 }
	 tPos += tIncre;
  }

  //create quad meshes
 

  int iPos = 0;
  int iHex = 0;
  for(int t = 0; t < tdiv; t++){
    for(int v = 0; v < vdiv; v++){
      for(int u = 0; u < udiv; u++){
	    TNR_HEX hex;
	    hex[0] = iPos;
	    hex[1] = iPos+1;
	    hex[2] = hex[0]+udiv+2;
	    hex[3] = hex[2] -1;
	    hex[4] = iPos + (udiv+1) * (vdiv+1);
	    hex[5] = hex[4]+1;
	    hex[6] = hex[5]+udiv+1;
	    hex[7] = hex[6] - 1;

	    mesh->setMesh(iHex,hex);
	    iPos ++;
	    iHex++;
	  }
	  iPos++;
    }
	iPos= iPos + udiv + 1;
  }
}
