#include "TNR_SHAPE.hxx"
#include "TNR_VECTOR.hxx"
#include "TNR_TRANSFORM.hxx"
#include "TNR_COMMON.hxx"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>



void  shape_make_rectangle(TNR_SECTION* shape,TNR_COMAMND* commands)
{

  double side  = shape->getDoubleValue("side_length",1.0);
  double height = shape->getDoubleValue("height_length",1.0);
  string leftBottom = shape->getStringValue("left_bottom","0.0 0.0 0.0");
  TNR_POINT leftBottomPos;

  sscanf(leftBottom.c_str(),"%lf %lf %lf",&(leftBottomPos[0]),&(leftBottomPos[1]),&(leftBottomPos[2]));

  

  TNR_SECTION* face = new TNR_SECTION("face",shape->getOwner());
  face->addKeyStringValue("name",shape->getStringValue("name",""));
  face->addKeyStringValue("surface_type","quadricSurface");
  face->addKeyStringValue("format","fff");

  TNR_POINT pos[8];
  //orgn
  pos[0][0]= leftBottomPos[0];  pos[0][1]= leftBottomPos[1];

  pos[1][0]=  pos[0][0] + 0.5* side;
  pos[1][1]=  pos[0][1] + 0.0;
  pos[2][0]=  pos[0][0] + side;
  pos[2][1]=  pos[0][1] + 0.0;
  pos[3][0]=  pos[0][0] + side;
  pos[3][1]=  pos[0][1] + 0.5 * height;
  pos[4][0]=  pos[0][0] + side;
  pos[4][1]=  pos[0][1] + height;
  pos[5][0]=  pos[0][0] + 0.5* side;
  pos[5][1]=  pos[0][1] + height;
  pos[6][0]=  pos[0][0];
  pos[6][1]=  pos[0][1] + height;
  pos[7][0]=  pos[0][0];
  pos[7][1]=  pos[0][1] + 0.5 * height;

  char buf[1024];
  for(int i = 0;i < 8;i++){
      sprintf(buf,"%f %f %f", pos[i][0], pos[i][1]);
      face->addData(buf);
  }

  shape->getOwner()->addSection(face);
}


void  shape_make_circle(TNR_SECTION* shape,TNR_COMAMND* commands)
{

  double radius  = shape->getDoubleValue("radius",1.0);
  string center  = shape->getStringValue("center","0.0 0.0 0,0");
  TNR_POINT centerPos;
  sscanf(center.c_str(),"%lf %lf %lf",&(centerPos[0]),&(centerPos[1]),&(centerPos[2]));


  string mesh_pattern  = commands->getStringValue("mesh_pattern","uniform");

  if(mesh_pattern == "nonuniform"){
    TNR_SECTION* face = new TNR_SECTION("face",shape->getOwner());
    face->addKeyStringValue("name",shape->getStringValue("name",""));
    face->addKeyStringValue("surface_type","quadricSurface");
    face->addKeyStringValue("edge[2]",shape->getStringValue("name",""));
	face->addKeyStringValue("format","fff");



    TNR_POINT pos[8];
    pos[0][0]=  0.5*radius*DCOS(45);
    pos[0][1]=  0.5*radius*DSIN(45);
    pos[1][0]=  pos[0][0];
    pos[1][1]=  0.0;
    pos[2][0]=  pos[0][0];
    pos[2][1]= -pos[0][1];
    pos[3][0]=  radius*3/4*DCOS(45);
    pos[3][1]= -radius*3/4*DSIN(45);
    pos[4][0]=  radius*DCOS(45);
    pos[4][1]= -radius*DSIN(45);
    pos[5][0]=  1.0;
    pos[5][1]=  0.0;
    pos[6][0]=  radius*DCOS(45);  
    pos[6][1]=  radius*DSIN(45);  
    pos[7][0]=  radius*3/4*DCOS(45);
    pos[7][1]=  radius*3/4*DSIN(45);
  
    char buf[1024];
    for(int i = 0;i < 8;i++){
		TNR_POINT p = pos[i]  + centerPos;
        sprintf(buf,"%f %f %f", p[0], p[1],p[2]);
        face->addData(buf);
    }

    shape->getOwner()->addSection(face);

    TNR_SECTION* edge = new TNR_SECTION("edge",shape->getOwner());
    edge->addKeyStringValue("name",shape->getStringValue("name",""));
    edge->addKeyStringValue("curve_type","circle");
    edge->addKeyStringValue("center",shape->getStringValue("center","0.0 0.0 0.0"));
	edge->addKeyStringValue("radius",shape->getStringValue("radius","1.0"));
	 
	shape->getOwner()->addSection(edge);


    TNR_SECTION* s0 = new TNR_SECTION;
    face->copy(*s0);
  
    TNR_section_transration(s0,string ("rotate"),string ("centerOfRotate"),centerPos,TNR_POINT (0.0,0.0,90.0));
    shape->getOwner()->addSection(s0);

    TNR_SECTION* s1 = new TNR_SECTION;
    s0->copy(*s1);
  
    TNR_section_transration(s1,string ("rotate"),string ("centerOfRotate"),centerPos,TNR_POINT (0.0,0.0,90.0));
    shape->getOwner()->addSection(s1);

    TNR_SECTION* s2 = new TNR_SECTION;
    s1->copy(*s2);
  
    TNR_section_transration(s2,string ("rotate"),string ("centerOfRotate"),centerPos,TNR_POINT (0.0,0.0,90.0));
    shape->getOwner()->addSection(s2);


    //center of shape(cube)
	TNR_SECTION* s3 = new TNR_SECTION("face",shape->getOwner());
	s3->addKeyStringValue("name",shape->getStringValue("name",""));
	s3->addKeyStringValue("surface_type","bilinearSurface");
    TNR_POINT positionOfCube[4];
    positionOfCube[0][0]=  -pos[2][0];
    positionOfCube[0][1]=   pos[2][1];
    positionOfCube[1]   =   pos[2];
    positionOfCube[2]   =   pos[0];
    positionOfCube[3][0]=  -pos[0][0];
    positionOfCube[3][1]=   pos[0][1];
  
    for(int i = 0;i < 4;i++){
	  TNR_POINT p = positionOfCube[i]  + centerPos;
      sprintf(buf,"%f %f %f", p[0], p[1],p[2]);
      s3->addData(buf);
    }
    shape->getOwner()->addSection(s3);
  }
  
  if(mesh_pattern == "uniform"){
    TNR_SECTION* face = new TNR_SECTION;
    face->setName(string("face"));
    face->addKeyStringValue("name",shape->getStringValue("name",""));
    face->addKeyStringValue("surface_type","circleSurface");
	face->addKeyStringValue("center",shape->getStringValue("center","0.0 0.0 0,0"));
	face->addKeyStringValue("radius",shape->getStringValue("radius","1.0"));
	face->addKeyStringValue("smooth","off");
	shape->getOwner()->addSection(face);
  }
}
 
void  shape_make_oval(TNR_SECTION* shape,TNR_COMAMND* commands)
{

  string mesh_pattern  = commands->getStringValue("mesh_pattern","uniform");

  if(mesh_pattern == "uniform"){
    TNR_SECTION* face = new TNR_SECTION;
    face->setName(string("face"));
    face->addKeyStringValue("name",shape->getStringValue("name",""));
    face->addKeyStringValue("surface_type","ellipseSurface");
	face->addKeyStringValue("center",shape->getStringValue("center","0.0,0.0,0.0"));
	face->addKeyStringValue("major_axis_radius",shape->getStringValue("major_axis_radius","1.0"));
	face->addKeyStringValue("minor_axis_radius",shape->getStringValue("minor_axis_radius","1.0"));
	face->addKeyStringValue("smooth","off");
	shape->getOwner()->addSection(face);
  }
  if(mesh_pattern == "nonuniform"){
	  TNR_ERR_write_message("oval:nonuniform is not support.");
  }
 
}